""" Objects pertaining to the various ingestion manifests """

from enum import Enum


class Telescope(Enum):
    """Codifying the names of our telescopes, because Janet can't abide magic strings"""

    VLA = 1
    EVLA = 2
    ALMA = 3
    VLBA = 4
    GBT = 5
    NONE = 6


class IngestionType(Enum):
    """Types of ingestion we'll have to do"""

    # ALMA products
    ALMA_SDM = Telescope.ALMA
    ALMA_CAL = Telescope.ALMA
    ALMA_AUDI = Telescope.ALMA

    # EVLA products
    EVLA_SDM = Telescope.EVLA
    EVLA_BDF = Telescope.EVLA
    EVLA_CAL = Telescope.EVLA

    # RealFast projects
    REALFAST_SDM = Telescope.EVLA

    # VLASS projects
    VLASS_QUICKLOOK = Telescope.EVLA

    # VLBA ingestion. (IDI and UVFITS products are treated the same.)
    VLBA_FITS = Telescope.VLBA

    # Coming Real Soon Now: VLBA Mark 4 product ingestion
    VLBA_MARK4 = Telescope.VLBA

    # Also coming Real Soon: GBT execution block ingestion
    GBT_EB = Telescope.GBT

    # Hot on its heels: LVLA execution block ingestion
    LVLA_EB = Telescope.VLA

    # When we just don't know what we're dealing with
    UNKNOWN = Telescope.NONE


class ScienceProductType(Enum):
    """Canonical collection of ingestible types of science products"""

    EXEC_BLOCK = "execution_block"
    CAL = "calibration"
    CATALOG = "catalog"
    IMAGE = "image"


class ScienceProduct:
    """Represents a science product in an ingestion manifest"""

    def __init__(self, sp_type: ScienceProductType, filename: str, locator: str, group_with: str):
        self.sp_type = sp_type
        self.filename = filename
        # product locator, used for input groups; locator for a known science product
        self.locator = locator
        # for "late" science products; they get added to an existing output group
        self.group_with = group_with


class AncillaryProductType:
    """The various types of ancillary products we'll encounter"""

    INGESTION_ARTIFACTS = "ingestion_artifacts"
    PIPELINE_ARTIFACTS = "pipeline_artifacts"
    PIPELINE_WEBLOG = "pipeline_weblog"
    LOG = "log_file"

    ### Images ###

    # our default FITS type
    FITS = "fits_image"

    VLASS_QUICKLOOK = "quicklook_rms_image"
    AUDI_FITS_MASK = "clean_mask"
    AUDI_PB_FITS = "primary_beam"
    ALPHA_FITS = "spectral_index"
    CANDIDATE_IMG = "candidate_image"
    THUMBNAIL_IMG = "thumbnail_image"


class AncillaryProduct:
    """Represents an ancillary product in an ingestion manifest"""

    def __init__(
        self, type: AncillaryProductType, filename: str, science_associate: str, group_with: str
    ):
        self.type = type
        self.filename = filename
        # make this an ancillary to a particular science product (assumes locator string)
        self.science_associate = science_associate  # TODO: enum?
        # make this an ancillary to the group of a science product (assumes locator string)
        self.group_with = group_with
